#include <stdio.h>
#include <pwd.h>
#include <stdarg.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/wait.h>
#include "popen2.h"

#define DEFAULT_USER "nobody"
#define PAM_SM_SESSION
#define MODULE_NAME "ssh_pam_agent"

#include <security/pam_modules.h>
#include <security/_pam_macros.h>

#define CMD_LEN 512

#define CMD "/bin/bash -c 'export HOME=%s;(keychain && . ~/.keychain/`uname -n`-sh && ssh-add) >/dev/null 2>&1'"

PAM_EXTERN
int pam_sm_open_session(pam_handle_t *pamh,int flags,int argc
			,const char **argv)
{
  const void *item = NULL;
  const char *user = NULL;
  struct passwd *pwent;
  int retval;
  char cmd[CMD_LEN];

  retval = pam_get_item(pamh, PAM_AUTHTOK, &item);
  if (retval != PAM_SUCCESS)
    return retval;

  if(!item)
    return PAM_AUTH_ERR;
  
  if ((retval = pam_get_user(pamh, &user, NULL)) != PAM_SUCCESS) {
    return retval;
  }
  
  if (!(user && (pwent = getpwnam(user)) && pwent->pw_dir &&
	*pwent->pw_dir))
    return PAM_AUTH_ERR;
  

  /* spawns a shell script (INSECURE) that 
   * adds the password to the ssh-agent */
  pid_t pid = fork();
  if(!pid) {
    setreuid(pwent->pw_uid, pwent->pw_uid);
    snprintf(cmd, CMD_LEN, CMD, pwent->pw_dir);
    
    FILE *in, *out;

    unsetenv("DISPLAY"); // ssh-agent looks it. it should not be set, but in any case...
    int res = popen2(cmd, &in, &out, &pid);
    if(res == -1) {
      perror("creating ssh-agent");
      exit(1);
    }

    fprintf(in, "%s\n", (char*)item);
    fclose(in);

    int status;
    waitpid(pid, &status, 0);

    exit(WEXITSTATUS(status));
  }
  int status;
  wait(&status);
  //if(WEXITSTATUS(status)) 
  //  debug("unsuccessfull"); 
  

  return PAM_SUCCESS;  
}

PAM_EXTERN
int pam_sm_close_session(pam_handle_t *pamh,int flags,int argc
			 ,const char **argv)
{
     return PAM_SUCCESS;
}

/* end of module definition */

#ifdef PAM_STATIC

/* static module data */

struct pam_module _pam_ssh_agent_modstruct = {
    "pam_ssh_agent",
    NULL,
    NULL,
    NULL,
    pam_sm_open_session,
    pam_sm_close_session,
    NULL,
};

#endif
