#include "popen2.h"
#include <unistd.h>

#include <sys/types.h>
#include <sys/socket.h>
#include <wait.h>

int popen2(const char* cmd, FILE **in, FILE **out, pid_t *child) {
  int in_socks[2];
  int out_socks[2];
  int res;
  res = socketpair(AF_UNIX, SOCK_STREAM, 0, in_socks);
  if(res == -1)
    return -1;
  
  res = socketpair(AF_UNIX, SOCK_STREAM, 0, out_socks);
  if(res == -1)
    return -1;

  *in = fdopen(in_socks[0], "w");
  *out = fdopen(out_socks[0], "r");

  pid_t pid = fork();
  if(pid == -1)
    return -1;
  if(pid == 0) {
    close(in_socks[0]);
    close(out_socks[0]);

    // redirect stdin/stdout through sockets
    dup2(in_socks[1], 0);
    dup2(out_socks[1], 1);
    
    // detach controlling terminal
    setsid(); 

    /* second fork allows:
     * 1) otherwise if the command opens a pty it would become it's controlling terminal.
     *    the fork fixes that
     * 2) allow us to wait the completion of the command in order to 'shutdown' the sockets 
     */
    pid_t wrapper = fork();
    if(wrapper == 0)
      execl("/bin/sh", "/bin/sh", "-c", cmd, 0);

    int status;
    waitpid(wrapper, &status, 0);

    // needed to generate an EOF on the other end
    // a close isn't enough
    shutdown(in_socks[1], SHUT_RDWR);
    shutdown(out_socks[1], SHUT_RDWR);
    exit(WEXITSTATUS(status));
  }

  if(child)
    *child = pid;

  return 0;
}
